using System;
using System.IO;
using System.Text;
using System.Linq;
using System.Threading;
using Raspberry.IO;
using Raspberry.IO.GeneralPurpose;
using System.Collections.Generic;

namespace Raspberry.IO.Components.Displays.Sda5708
{
	public sealed class Sda5708Font
	{
		public static readonly Dictionary<char, string[]> Patterns = new Dictionary<char, string[]>
		{
			﻿{ ' ', new[]{ "     ",
              			  "     ",
              			  "     ",
              			  "     ",
              		      "     ",
              			  "     ",
              			  "     " }},

			{ '!', new[]{ "  1  ",
			              "  1  ",
			              "  1  ",
			              "  1  ",
			              "  1  ",
			              "     ",
			              "  1  " }},

			{ '"', new[]{ " 1 1 ",
			              " 1 1 ",
			              " 1 1 ",
			              "     ",
			              "     ",
			              "     ",
			              "     " }},

			{ '#', new[]{ " 1 1 ",
			              " 1 1 ",
			              "11111",
			              " 1 1 ",
			              "11111",
			              " 1 1 ",
			              " 1 1 " }},

			{ '$', new[]{ "  1  ",
			              " 1111",
			              "1 1  ",
			              " 111 ",
			              "  1 1",
			              "1111 ",
			              "  1  " }},

			{ '%', new[]{ "11   ",
			              "11  1",
			              "   1 ",
			              "  1  ",
			              " 1   ",
			              "1  11",
			              "   11" }},

			{ '&', new[]{ " 11  ",
			              "1  1 ",
			              "1 1  ",
			              " 1   ",
			              "1 1 1",
			              "1  1 ",
			              " 11 1" }},

			{ '\'', new[]{ " 11  ",
			              "  1  ",
			              " 1   ",
			              "     ",
			              "     ",
			              "     ",
			              "     " }},

			{ '(', new[]{ "   1 ",
			              "  1  ",
			              " 1   ",
			              " 1   ",
			              " 1   ",
			              "  1  ",
			              "   1 " }},

			{ ')', new[]{ " 1   ",
			              "  1  ",
			              "   1 ",
			              "   1 ",
			              "   1 ",
			              "  1  ",
			              " 1   " }},

			{ '*', new[]{ "     ",
			              " 1 1 ",
			              "  1  ",
			              "11111",
			              "  1  ",
			              " 1 1 ",
			              "     " }},

			{ '+', new[]{ "     ",
			              "  1  ",
			              "  1  ",
			              "11111",
			              "  1  ",
			              "  1  ",
			              "     " }},

			{ ',', new[]{ "     ",
			              "     ",
			              "     ",
			              "     ",
			              " 11  ",
			              "  1  ",
			              " 1   " }},

			{ '-', new[]{ "     ",
			              "     ",
			              "     ",
			              "11111",
			              "     ",
			              "     ",
			              "     " }},

			{ '.', new[]{ "     ",
			              "     ",
			              "     ",
			              "     ",
			              "     ",
			              " 11  ",
			              " 11  " }},

			{ '/', new[]{ "     ",
			              "    1",
			              "   1 ",
			              "  1  ",
			              " 1   ",
			              "1    ",
			              "     " }},

			{ '0', new[]{ " 111 ",
			              "1   1",
			              "1  11",
			              "1 1 1",
			              "11  1",
			              "1   1",
			              " 111 " }},

			{ '1', new[]{ "  1  ",
			              " 11  ",
			              "  1  ",
			              "  1  ",
			              "  1  ",
			              "  1  ",
			              " 111 " }},

			{ '2', new[]{ " 111 ",
			              "1   1",
			              "    1",
			              "   1 ",
			              "  1  ",
			              " 1   ",
			              "11111" }},

			{ '3', new[]{ "11111",
			              "   1 ",
			              "  1  ",
			              "   1 ",
			              "    1",
			              "1   1",
			              " 111 " }},

			{ '4', new[]{ "   1 ",
			              "  11 ",
			              " 1 1 ",
			              "1  1 ",
			              "11111",
			              "   1 ",
			              "   1 " }},

			{ '5', new[]{ "11111",
			              "1    ",
			              "1111 ",
			              "    1",
			              "    1",
			              "1   1",
			              " 111 " }},

			{ '6', new[]{ "  11 ",
			              " 1   ",
			              "1    ",
			              "1111 ",
			              "1   1",
			              "1   1",
			              " 111 " }},

			{ '7', new[]{ "11111",
			              "    1",
			              "   1 ",
			              "  1  ",
			              " 1   ",
			              " 1   ",
			              " 1   " }},

			{ '8', new[]{ " 111 ",
			              "1   1",
			              "1   1",
			              " 111 ",
			              "1   1",
			              "1   1",
			              " 111 " }},

			{ '9', new[]{ " 111 ",
			              "1   1",
			              "1   1",
			              " 1111",
			              "    1",
			              "   1 ",
			              " 11  " }},

			{ ':', new[]{ "     ",
			              " 11  ",
			              " 11  ",
			              "     ",
			              " 11  ",
			              " 11  ",
			              "     " }},

			{ ';', new[]{ "     ",
			              " 11  ",
			              " 11  ",
			              "     ",
			              " 11  ",
			              "  1  ",
			              " 1   " }},

			{ '<', new[]{ "    1",
			              "   1 ",
			              "  1  ",
			              " 1   ",
			              "  1  ",
			              "   1 ",
			              "    1" }},

			{ '=', new[]{ "     ",
			              "     ",
			              "11111",
			              "     ",
			              "11111",
			              "     ",
			              "     " }},

			{ '>', new[]{ "1    ",
			              " 1   ",
			              "  1  ",
			              "   1 ",
			              "  1  ",
			              " 1   ",
			              "1    " }},

			{ '?', new[]{ " 111 ",
			              "1   1",
			              "    1",
			              "   1 ",
			              "  1  ",
			              "     ",
			              "  1  " }},

			{ '@', new[]{ " 111 ",
			              "1   1",
			              "    1",
			              " 11 1",
			              "1 1 1",
			              "1 1 1",
			              " 111 " }},

			{ 'A', new[]{ " 111 ",
			              "1   1",
			              "1   1",
			              "1   1",
			              "11111",
			              "1   1",
			              "1   1" }},

			{ 'Ä', new[]{ "1   1",
					      "     ",
						  " 111 ",
						  "1   1",
						  "11111",
						  "1   1",
						  "1   1" }},

			{ 'B', new[]{ "1111 ",
			              "1   1",
			              "1   1",
			              "1111 ",
			              "1   1",
			              "1   1",
			              "1111 " }},

			{ 'C', new[]{ " 111 ",
			              "1   1",
			              "1    ",
			              "1    ",
			              "1    ",
			              "1   1",
			              " 111 " }},

			{ 'D', new[]{ "111  ",
			              "1  1 ",
			              "1   1",
			              "1   1",
			              "1   1",
			              "1  1 ",
			              "111  " }},

			{ 'E', new[]{ "11111",
			              "1    ",
			              "1    ",
			              "1111 ",
			              "1    ",
			              "1    ",
			              "11111" }},

			{ 'F', new[]{ "11111",
			              "1    ",
			              "1    ",
			              "111  ",
			              "1    ",
			              "1    ",
			              "1    " }},

			{ 'G', new[]{ " 111 ",
			              "1   1",
			              "1    ",
			              "1    ",
			              "1  11",
			              "1   1",
			              " 111 " }},

			{ 'H', new[]{ "1   1",
			              "1   1",
			              "1   1",
			              "11111",
			              "1   1",
			              "1   1",
			              "1   1" }},

			{ 'I', new[]{ " 111 ",
			              "  1  ",
			              "  1  ",
			              "  1  ",
			              "  1  ",
			              "  1  ",
			              " 111 " }},

			{ 'J', new[]{ "  111",
			              "   1 ",
			              "   1 ",
			              "   1 ",
			              "   1 ",
			              "1  1 ",
			              " 11  " }},

			{ 'K', new[]{ "1   1",
			              "1  1 ",
			              "1 1  ",
			              "11   ",
			              "1 1  ",
			              "1  1 ",
			              "1   1" }},

			{ 'L', new[]{ "1    ",
			              "1    ",
			              "1    ",
			              "1    ",
			              "1    ",
			              "1    ",
			              "11111" }},

			{ 'M', new[]{ "1   1",
			              "11 11",
			              "1 1 1",
			              "1   1",
			              "1   1",
			              "1   1",
			              "1   1" }},

			{ 'N', new[]{ "1   1",
			              "1   1",
			              "11  1",
			              "1 1 1",
			              "1  11",
			              "1   1",
			              "1   1" }},

			{ 'O', new[]{ " 111 ",
			              "1   1",
			              "1   1",
			              "1   1",
			              "1   1",
			              "1   1",
			              " 111 " }},

			{ 'Ö', new[]{ "1   1",
					      "     ",
						  " 111 ",
						  "1   1",
						  "1   1",
						  "1   1",
						  " 111 " }},

			{ 'P', new[]{ "1111 ",
			              "1   1",
			              "1   1",
			              "1111 ",
			              "1    ",
			              "1    ",
			              "1    " }},

			{ 'Q', new[]{ " 111 ",
			              "1   1",
			              "1   1",
			              "1   1",
			              "1 1 1",
			              "1  1 ",
			              " 11 1" }},

			{ 'R', new[]{ "1111 ",
			              "1   1",
			              "1   1",
			              "1111 ",
			              "1 1  ",
			              "1  1 ",
			              "1   1" }},

			{ 'S', new[]{ " 1111",
			              "1    ",
			              "1    ",
			              " 111 ",
			              "    1",
			              "    1",
			              "1111 " }},

			{ 'T', new[]{ "11111",
			              "  1  ",
			              "  1  ",
			              "  1  ",
			              "  1  ",
			              "  1  ",
			              "  1  " }},

			{ 'U', new[]{ "1   1",
			              "1   1",
			              "1   1",
			              "1   1",
			              "1   1",
			              "1   1",
			              " 111 " }},

			{ 'Ü', new[]{ "1   1",
						  "     ",
						  "1   1",
						  "1   1",
						  "1   1",
						  "1   1",
						  " 111 " }},

			{ 'V', new[]{ "1   1",
			              "1   1",
			              "1   1",
			              "1   1",
			              "1   1",
			              " 1 1 ",
			              "  1  " }},

			{ 'W', new[]{ "1   1",
			              "1   1",
			              "1   1",
			              "1 1 1",
			              "1 1 1",
			              "11 11",
			              "1   1" }},

			{ 'X', new[]{ "1   1",
			              "1   1",
			              " 1 1 ",
			              "  1  ",
			              " 1 1 ",
			              "1   1",
			              "1   1" }},

			{ 'Y', new[]{ "1   1",
			              "1   1",
			              " 1 1 ",
			              "  1  ",
			              "  1  ",
			              "  1  ",
			              "  1  " }},

			{ 'Z', new[]{ "11111",
			              "    1",
			              "   1 ",
			              "  1  ",
			              " 1   ",
			              "1    ",
			              "11111" }},

			{ '[', new[]{ "  111",
			              "  1  ",
			              "  1  ",
			              "  1  ",
			              "  1  ",
			              "  1  ",
			              "  111" }},

			{ '\\', new[]{ "     ",
			              "1    ",
			              " 1   ",
			              "  1  ",
			              "   1 ",
			              "    1",
			              "     " }},

			{ ']', new[]{ "111  ",
			              "  1  ",
			              "  1  ",
			              "  1  ",
			              "  1  ",
			              "  1  ",
			              "111  " }},

			{ '^', new[]{ "  1  ",
			              " 1 1 ",
			              "1   1",
			              "     ",
			              "     ",
			              "     ",
			              "     " }},

			{ '_', new[]{ "     ",
			              "     ",
			              "     ",
			              "     ",
			              "     ",
			              "     ",
			              "11111" }},

			{ '`', new[]{ " 1   ",
			              "  1  ",
			              "   1 ",
			              "     ",
			              "     ",
			              "     ",
			              "     " }},

			{ 'a', new[]{ "     ",
			              "     ",
			              " 111 ",
			              "    1",
			              " 1111",
			              "1   1",
			              " 1111" }},

			{ 'ä', new[]{ " 1 1 ",
						  "     ",
						  " 111 ",
						  "    1",
						  " 1111",
						  "1   1",
						  " 1111" }},

			{ 'b', new[]{ "1    ",
			              "1    ",
			              "1 11 ",
			              "11  1",
			              "1   1",
			              "1   1",
			              "1111 " }},

			{ 'c', new[]{ "     ",
			              "     ",
			              " 111 ",
			              "1    ",
			              "1    ",
			              "1   1",
			              " 111 " }},

			{ 'd', new[]{ "    1",
			              "    1",
			              " 11 1",
			              "1  11",
			              "1   1",
			              "1   1",
			              " 1111" }},

			{ 'e', new[]{ "     ",
			              "     ",
			              " 111 ",
			              "1   1",
			              "11111",
			              "1    ",
			              " 111 " }},

			{ 'f', new[]{ "  11 ",
			              " 1  1",
			              " 1   ",
			              "111  ",
			              " 1   ",
			              " 1   ",
			              " 1   " }},

			{ 'g', new[]{ "     ",
			              "     ",
			              " 1111",
			              "1   1",
			              " 1111",
			              "    1",
			              "  11 " }},

			{ 'h', new[]{ "1    ",
			              "1    ",
			              "1 11 ",
			              "11  1",
			              "1   1",
			              "1   1",
			              "1   1" }},

			{ 'i', new[]{ "  1  ",
			              "     ",
			              " 11  ",
			              "  1  ",
			              "  1  ",
			              "  1  ",
			              " 111 " }},

			{ 'j', new[]{ "   1 ",
			              "     ",
			              "  11 ",
			              "   1 ",
			              "   1 ",
			              "1  1 ",
			              " 11  " }},

			{ 'k', new[]{ " 1   ",
			              " 1   ",
			              " 1  1",
			              " 1 1 ",
			              " 11  ",
			              " 1 1 ",
			              " 1  1" }},

			{ 'l', new[]{ " 11  ",
			              "  1  ",
			              "  1  ",
			              "  1  ",
			              "  1  ",
			              "  1  ",
			              " 111 " }},

			{ 'm', new[]{ "     ",
			              "     ",
			              "11 1 ",
			              "1 1 1",
			              "1 1 1",
			              "1   1",
			              "1   1" }},

			{ 'n', new[]{ "     ",
			              "     ",
			              "1 11 ",
			              "11  1",
			              "1   1",
			              "1   1",
			              "1   1" }},

			{ 'o', new[]{ "     ",
			              "     ",
			              " 111 ",
			              "1   1",
			              "1   1",
			              "1   1",
			              " 111 " }},

			{ 'ö', new[]{ " 1 1 ",
						  "     ",
						  " 111 ",
						  "1   1",
						  "1   1",
						  "1   1",
						  " 111 " }},

			{ 'p', new[]{ "     ",
			              "     ",
			              "1111 ",
			              "1   1",
			              "1111 ",
			              "1    ",
			              "1    " }},

			{ 'q', new[]{ "     ",
			              "     ",
			              " 11 1",
			              "1  11",
			              " 1111",
			              "    1",
			              "    1" }},

			{ 'r', new[]{ "     ",
			              "     ",
			              "1 11 ",
			              "11  1",
			              "1    ",
			              "1    ",
			              "1    " }},

			{ 's', new[]{ "     ",
			              "     ",
			              " 111 ",
			              "1    ",
			              " 111 ",
			              "    1",
			              "1111 " }},

			{ 't', new[]{ " 1   ",
			              " 1   ",
			              "111  ",
			              " 1   ",
			              " 1   ",
			              " 1  1",
			              "  11 " }},

			{ 'u', new[]{ "     ",
			              "     ",
			              "1   1",
			              "1   1",
			              "1   1",
			              "1  11",
			              " 11 1" }},

			{ 'ü', new[]{ " 1 1 ",
						  "     ",
						  "1   1",
						  "1   1",
						  "1   1",
						  "1  11",
						  " 11 1" }},

			{ 'v', new[]{ "     ",
			              "     ",
			              "1   1",
			              "1   1",
			              "1   1",
			              " 1 1 ",
			              "  1  " }},

			{ 'w', new[]{ "     ",
			              "     ",
			              "1   1",
			              "1   1",
			              "1 1 1",
			              "1 1 1",
			              " 1 1 " }},

			{ 'x', new[]{ "     ",
			              "     ",
			              "1   1",
			              " 1 1 ",
			              "  1  ",
			              " 1 1 ",
			              "1   1" }},

			{ 'y', new[]{ "     ",
			              "     ",
			              "1   1",
			              "1   1",
			              " 1111",
			              "    1",
			              " 111 " }},

			{ 'z', new[]{ "     ",
			              "     ",
			              "11111",
			              "   1 ",
			              "  1  ",
			              " 1   ",
			              "11111" }},

			{ 'ß', new[]{ " 11  ",
					      "1  1 ",
					      "111  ",
					      "1  1 ",
					      "1  1 ",
					      "111  ",
					      "1    " }},

			{ '{', new[]{ "   1 ",
			              "  1  ",
			              "  1  ",
			              " 1   ",
			              "  1  ",
			              "  1  ",
			              "   1 " }},

			{ '|', new[]{ "  1  ",
			              "  1  ",
			              "  1  ",
			              "  1  ",
			              "  1  ",
			              "  1  ",
			              "  1  " }},

			{ '}', new[]{ " 1   ",
			              "  1  ",
			              "  1  ",
			              "   1 ",
			              "  1  ",
			              "  1  ",
			              " 1   " }},
		};
	}
}